import platform

from ajenti.api import *
from ajenti.api.sensors import Sensor


@plugin
class HostnameSensor (Sensor):
    id = 'hostname'
    timeout = 60

    def measure(self, variant=None):
        return platform.node()



"""
Module for sending usage statistics to ajenti.org
"""

import ajenti
import requests
import json
import gevent
import logging

from ajenti.util import *


HOST = 'meta.ajenti.org'
URL = 'http://%s/api/v2/' % HOST


enabled = True


@public
def start():
    gevent.spawn(worker)


def send(url, data):
    id = ajenti.config.tree.installation_id
    if id:
        data['id'] = id
    logging.debug('Feedback >> %s (%s)' % (url, data))
    try:
        response = requests.post(URL + url, data=data)
    except:
        raise IOError()
    logging.debug('Feedback << %s' % response.content)
    return json.loads(response.content)


@public
def send_feedback(email, text):
    send('feedback', {'email': email, 'text': text})


def worker():
    global enabled
    enabled = ajenti.config.tree.enable_feedback
    if enabled:
        if not ajenti.config.tree.installation_id:
            logging.debug('Registering installation')
            enabled = False
            try:
                data = {
                    'version': ajenti.version,
                    'os': ajenti.platform,
                    'edition': ajenti.edition,
                }
                resp = send('register', data)
                if resp['status'] != 'ok':
                    return
            except IOError:
                pass
            ajenti.config.tree.installation_id = resp['id']
            ajenti.config.save()
            enabled = True

        while True:
            try:
                send('ping', {})
            except IOError:
                pass
            gevent.sleep(3600 * 12)



# coding=utf-8
from ajenti.api import plugin, BasePlugin
from ajenti.plugins.configurator.api import ClassConfigEditor

import manager


@plugin
class OpenVPNClassConfigEditor (ClassConfigEditor):
    title = 'OpenVPN'
    icon = 'globe'

    def init(self):
        self.append(self.ui.inflate('openvpn:config'))


@plugin
class OpenVPNBackend (BasePlugin):
    """
    OpenVPN plugin and widget backend
    @ivar _m: manager instance
    """
    default_classconfig = {'addr': '', 'password': ''}
    classconfig_editor = OpenVPNClassConfigEditor

    def setup(self):
        """
        Initializes instance variables, creates manager instance and tests connection
        @return: Nothing
        """
        # Retrieve configuration settings
        addr = self.classconfig['addr']
        password = self.classconfig['password']
        if not addr:
            raise Exception('No address specified')
        # Check if addr is a "host:port" string
        (host, colon, port) = addr.partition(":")
        try:
            if colon:
                # IPv4/IPv6
                self._m = manager.manager(host=host, port=int(port), password=password)
            else:
                # UNIX Socket
                self._m = manager.manager(path=addr, password=password)
            # Test connection
            self.getpid()
        except Exception, e:
            raise Exception(str(e))

    def _execute(self, func, *args):
        """
        OpenVPN management interface is unable to handle more than one connection.
        This method is used to connect and disconnect (if was not connected manually) every
        time plugin wants to execute a command.
        @param func: Function to call
        @param args: Argument List
        @return: Function result
        """
        if not self._m.connected:
            self._m.connect()
            result = self._execute(func, *args)
            self._m.disconnect()
        else:
            result = func(*args)
        return result

    def connect(self):
        self._m.connect()

    def disconnect(self):
        self._m.disconnect()

    def getstatus(self):
        return self._execute(self._m.status)

    def getstats(self):
        return self._execute(self._m.stats)

    def killbyaddr(self, addr):
        return self._execute(self._m.killbyaddr, addr)

    def restartcond(self):
        return self._execute(self._m.signal, "SIGUSR1")

    def restarthard(self):
        return self._execute(self._m.signal, "SIGHUP")

    def getmessages(self):
        return self._execute(self._m.messages)

    def getpid(self):
        return self._execute(self._m.pid)



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='Processes',
    icon='th-list',
    dependencies=[
        PluginDependency('main'),
    ],
)


def init():
    import main



from ajenti.api import *
from ajenti.plugins import *


info = PluginInfo(
    title='Apache',
    icon='globe',
    dependencies=[
        PluginDependency('webserver_common'),
        BinaryDependency('apache2'),
    ],
)


def init():
    import main



import os

from ajenti.api import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import on
from ajenti.ui.binder import Binder
from ajenti.util import platform_select

from reconfigure.configs import BIND9Config
from reconfigure.items.bind9 import ZoneData


@plugin
class BIND9Plugin (SectionPlugin):
    config_root = platform_select(
        debian='/etc/bind/',
        centos='/etc/named/',
    )
    
    config_path = platform_select(
        debian='/etc/bind/named.conf',
        centos='/etc/named.conf',
    )

    def init(self):
        self.title = 'BIND9'
        self.icon = 'globe'
        self.category = _('Software')

        self.append(self.ui.inflate('bind9:main'))

        self.config = BIND9Config(path=self.config_path)
        self.binder = Binder(None, self)
        self.find('zones').new_item = lambda c: ZoneData()

        def post_zone_bind(o, c, i, u):
            path = i.file
            if not path.startswith('/'):
                path = self.config_root + path
            exists = os.path.exists(path)
            u.find('no-file').visible = not exists
            u.find('file').visible = exists
            if exists:
                u.find('editor').value = open(path).read()

            def on_save_zone():
                open(path, 'w').write(u.find('editor').value)
                self.context.notify('info', _('Zone saved'))

            def on_create_zone():
                open(path, 'w').write("""$TTL    604800
@       IN      SOA     ns. root.ns. (
                              1         ; Serial
                         604800         ; Refresh
                          86400         ; Retry
                        2419200         ; Expire
                         604800 )       ; Negative Cache TTL
;
@                   IN      NS      ns.
example.com.        IN      A       127.0.0.1
example.com.        IN      AAAA    ::1
""")
                post_zone_bind(o, c, i, u)

            u.find('save-zone').on('click', on_save_zone)
            u.find('create-zone').on('click', on_create_zone)
        self.find('zones').post_item_bind = post_zone_bind

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.config.load()
        self.binder.reset(self.config.tree).autodiscover().populate()

    @on('save', 'click')
    def save(self):
        self.binder.update()
        self.config.save()
        self.refresh()
        self.context.notify('info', _('Saved'))



import psutil

from ajenti.api import *
from ajenti.ui import *


@plugin
class NetworkManager (BasePlugin):
    def get_devices(self):
        return psutil.network_io_counters(pernic=True).keys()


@interface
class INetworkConfig (object):
    interfaces = {}

    @property
    def interface_list(self):
        return self.interfaces.values()

    def rescan(self):
        pass

    def save(self):
        pass


@interface
class INetworkConfigBit (object):
    def apply(self):
        pass


@plugin
class NetworkConfigBit (UIElement, INetworkConfigBit):
    cls = 'unknown'
    iface = None
    title = 'Unknown'
    typeid = 'box'


class NetworkInterface(object):
    def __init__(self):
        self.up = False
        self.auto = False
        self.name = ''
        self.devclass = ''
        self.addressing = 'static'
        self.bits = []
        self.params = {'address': '0.0.0.0'}
        self.type = ''
        self.editable = True

    def __getitem__(self, idx):
        if idx in self.params:
            return self.params[idx]
        else:
            return ''

    def __setitem__(self, idx, val):
        self.params[idx] = val

    def add_bits(self, ui):
        for cls in INetworkConfigBit.get_classes():
            if cls.cls in self.bit_classes:
                b = cls.new(ui)
                b.iface = self
                b.refresh()
                self.bits.append(b)



from ajenti.api import plugin
from ajenti.api.sensors import Sensor
from ajenti.plugins.dashboard.api import DashboardWidget
from ajenti.util import str_timedelta


@plugin
class UptimeSensor (Sensor):
    id = 'uptime'
    timeout = 1

    def measure(self, variant):
        return float(open('/proc/uptime').read().split()[0])


@plugin
class UptimeWidget (DashboardWidget):
    name = _('Uptime')
    icon = 'off'

    def init(self):
        self.sensor = Sensor.find('uptime')
        self.append(self.ui.inflate('sensors:value-widget'))

        self.find('icon').text = 'off'
        self.find('name').text = 'Uptime'
        self.find('value').text = str_timedelta(self.sensor.value())



import traceback
import subprocess
import logging
import time
import sys
import fcntl
import locale
import os
import catcher

import ajenti


def public(f):
    """"
    Use a decorator to avoid retyping function/class names.

    Based on an idea by Duncan Booth:
    http://groups.google.com/group/comp.lang.python/msg/11cbb03e09611b8a

    Improved via a suggestion by Dave Angel:
    http://groups.google.com/group/comp.lang.python/msg/3d400fb22d8a42e1

    """
    all = sys.modules[f.__module__].__dict__.setdefault('__all__', [])
    if f.__name__ not in all:  # Prevent duplicates if run from an IDE.
        all.append(f.__name__)
    return f
public(public)  # Emulate decorating ourself


@public
def str_fsize(sz):
    """
    Formats file size as string (i.e., 1.2 Mb)
    """
    if sz < 1024:
        return '%.1f bytes' % sz
    sz /= 1024.0
    if sz < 1024:
        return '%.1f Kb' % sz
    sz /= 1024.0
    if sz < 1024:
        return '%.1f Mb' % sz
    sz /= 1024.0
    return '%.1f Gb' % sz


@public
def str_timedelta(s):
    """
    Formats a time delta (i.e., "5 days, 5:06:07")
    """
    d60 = lambda x: ('0' if (x % 60) < 10 else '') + str(x % 60)
    s = int(s)
    r = '%s:%s:%s' % (d60(s / 3600 % 24), d60(s / 60), d60(s))
    s /= 3600 * 24
    if s > 0:
        r = '%i days, ' % s + r
    return r


@public
def cache_value(duration):
    """
    Makes a function lazy.

    :param duration: cache duration in seconds
    """
    def decorator(fx):
        fx.__cached = None
        fx.__cached_at = 0

        def wrapper(*args, **kwargs):
            if time.time() - fx.__cached_at > duration:
                val = fx(*args, **kwargs)
                fx.__cached = val
                fx.__cached_at = time.time()
            else:
                val = fx.__cached
            return val
        wrapper.__doc__ = fx.__doc__
        return wrapper
    return decorator


@public
def platform_select(**values):
    """
    Selects a value from **kwargs depending on runtime platform::

        service = platform_select(
            debian='samba',
            ubuntu='smbd',
            centos='smbd',
            default='samba',
        )
    """
    if ajenti.platform_unmapped in values:
        return values[ajenti.platform_unmapped]    
    if ajenti.platform in values:
        return values[ajenti.platform]
    return values.get('default', None)


@public
def make_report(e):
    """
    Formats a bug report.
    """
    import platform as _platform
    from ajenti.plugins import manager
    from ajenti import platform, platform_unmapped, platform_string, installation_uid, version, debug

    # Finalize the reported log
    logging.blackbox.stop()

    tb = traceback.format_exc(e)
    
    catcher_url = None
    try:
        report = catcher.collect(e)
        html = catcher.formatters.HTMLFormatter().format(report, maxdepth=3)
        catcher_url = catcher.uploaders.AjentiOrgUploader().upload(html)
    except:
        pass

    return """Ajenti %s bug report
--------------------
Detected platform: %s / %s / %s
Architecture: %s
Python: %s
Installation: %s
Debug: %s
Locale: %s
Loaded plugins:
%s

%s
%s

Log content:
%s
            """ % (
        version,
        platform, platform_unmapped, platform_string,
        subprocess.check_output(['uname', '-mp']),
        '.'.join([str(x) for x in _platform.python_version_tuple()]),
        installation_uid,
        debug,
        locale.getlocale(locale.LC_MESSAGES),
        ' '.join(manager.get_order()),
        tb,
        catcher_url or 'Failed to upload traceback',
        logging.blackbox.buffer,
    )


@public
class PidFile(object):
    """
    Context manager that locks a pid file.  Implemented as class
    not generator because daemon.py is calling .__exit__() with no parameters
    instead of the None, None, None specified by PEP-343.
    """

    def __init__(self, path):
        self.path = path
        self.pidfile = None

    def __enter__(self):
        self.pidfile = open(self.path, "a+")
        try:
            fcntl.flock(self.pidfile.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
        except IOError:
            raise SystemExit("Already running according to " + self.path)
        self.pidfile.seek(0)
        self.pidfile.truncate()
        self.pidfile.write(str(os.getpid()))
        self.pidfile.flush()
        self.pidfile.seek(0)
        return self.pidfile

    def __exit__(self, exc_type=None, exc_value=None, exc_tb=None):
        try:
            self.pidfile.close()
        except IOError as err:
            # ok if file was just closed elsewhere
            if err.errno != 9:
                raise
        os.remove(self.path)



from ajenti.api import *
from ajenti.plugins import *
from ajenti.ui import *


info = PluginInfo(
    title=_('CodeMirror code editor'),
    icon='link',
    dependencies=[
        PluginDependency('main')
    ],
)


def init():

    @p('width', default='auto')
    @p('height', default='auto')
    @p('mode', default='default', type=unicode)
    @p('value', default='', type=unicode, bindtypes=[str, unicode])
    @plugin
    class CodeArea (UIElement):
        typeid = 'codearea'



from ajenti.api import *
from ajenti.api.sensors import *
from ajenti.plugins.main.api import SectionPlugin
from ajenti.ui import *
from ajenti.ui.binder import Binder

from api import *


@plugin
class NetworkPlugin (SectionPlugin):
    platforms = ['debian', 'centos']

    def init(self):
        self.title = _('Network')
        self.icon = 'globe'
        self.category = _('System')
        self.net_config = INetworkConfig.get()

        self.append(self.ui.inflate('network:main'))

        def post_interface_bind(o, c, i, u):
            i.add_bits(self.ui)
            for bit in i.bits:
                u.find('bits').append(self.ui.create(
                    'tab',
                    children=[bit],
                    title=bit.title,
                ))
            u.find('up').on('click', self.on_up, i)
            u.find('down').on('click', self.on_down, i)
            u.find('restart').on('click', self.on_restart, i)

        def post_interface_update(o, c, i, u):
            for bit in i.bits:
                bit.apply()

        self.find('interfaces').post_item_bind = post_interface_bind
        self.find('interfaces').post_item_update = post_interface_update

        self.binder = Binder(self.net_config, self)

    def on_page_load(self):
        self.refresh()

    def refresh(self):
        self.net_config.rescan()

        sensor = Sensor.find('traffic')
        for i in self.net_config.interface_list:
            i.tx, i.rx = sensor.value(i.name)

        self.binder.reset().autodiscover().populate()
        return

    def on_up(self, iface=None):
        self.net_config.up(iface)
        self.refresh()

    def on_down(self, iface=None):
        self.net_config.down(iface)
        self.refresh()

    def on_restart(self, iface=None):
        self.on_down(iface)
        self.on_up(iface)

    @on('save', 'click')
    def on_save(self):
        self.binder.update()
        self.net_config.save()
        self.refresh()
        self.context.notify('info', _('Saved'))
